unit UAutoCAD;

interface

uses
  Windows, SysUtils, Messages, Variants, Classes, Registry, ComObj, ActiveX,
  AutoCAD_TLB;

resourcestring
  rsDefClassName = 'AutoCAD.Application';

type
  TPoint3D = record
    X, Y, Z: Double;
  end;
  
  procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
      const acVisible: Boolean);
  procedure AcShow(WndState: AcWindowState = acMax);
  procedure AcHide;
  procedure AcRelease;
  function EnumAutoCAD(Items: TStrings): Boolean;
  function GetCurrAutoCADClassName: String;
  function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
  function EnumAutoCADHardcopy(Items: TStrings): Boolean;

  function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
  function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant; overload;
  function AcadPoint(APoint: TPoint3D): OleVariant; overload;

var
  Acad: OleVariant;
  
implementation

resourcestring
  rsAcadNotFound = 'AutoCAD  !';
  rsAutoCADRegPath = '\SOFTWARE\Autodesk\AutoCAD';

procedure AcConnect(const acClassName: String; const acNewInstance: Boolean;
    const acVisible: Boolean);
var
  IU: IUnknown;
  ClassID: TCLSID;
  IsCreate: Boolean;
  ErrorCode: HResult;
begin
  if VarIsClear(Acad) then //     
  begin
    //  AutoCAD     
    ErrorCode:= CLSIDFromProgID(PWideChar(WideString(acClassName)), ClassID);
    if not Succeeded(ErrorCode) then
      raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
    //       AutoCAD
    IsCreate:= acNewInstance or (not Succeeded(GetActiveObject(ClassID, nil, IU)));
    if IsCreate then
      try
        //    AutoCAD
        Acad:= CreateComObject(ClassID) as IDispatch;
      except
        raise EOleSysError.Create(rsAcadNotFound, ErrorCode, 0);
      end
    else
      Acad:= IU as IDispatch;
    //   AutoCAD
    if not VarIsClear(Acad) then Acad.Visible:= acVisible;
  end;
end;

procedure AcShow(WndState: AcWindowState = acMax);
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Visible:= True;
    Acad.WindowState:= WndState;
  end;
end;

procedure AcHide;
begin
  if not VarIsClear(Acad) then Acad.Visible:= False;
end;

procedure AcRelease;
begin
  if not VarIsClear(Acad) then
  begin
    Acad.Quit;
    Acad:= Unassigned;
  end;
end;

function EnumAutoCAD(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly(rsAutoCADRegPath);
      if not Result then Exit;
      GetKeyNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function GetCurrAutoCADClassName: String;
var
  reg: TRegistry;
  res: Boolean;
begin
  Result:= EmptyStr;
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_CURRENT_USER;
      res:= OpenKeyReadOnly('\Software\Classes\' + rsDefClassName + '\CurVer');
      if not res then Exit;
      Result:= ReadString('');
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function AutoCADFamilyNameToClassName(const AFamilyName: String): String;
var
  p: Integer;
begin
  Result:= AFamilyName;
  p:= Pos('.', Result);
  if Copy(Result, p, MaxInt) = '.0' then
    Delete(Result, p, MaxInt);
  Result:= StringReplace(Result, 'R', rsDefClassName + '.', [rfIgnoreCase]);
end;

function EnumAutoCADHardcopy(Items: TStrings): Boolean;
var
  reg: TRegistry;
begin
  reg:= TRegistry.Create;
  try
    with reg do
    begin
      RootKey:= HKEY_LOCAL_MACHINE;
      Result:= OpenKeyReadOnly('SOFTWARE\Autodesk\Hardcopy');
      if not Result then Exit;
      GetValueNames(Items);
    end;
  finally
    FreeAndNil(reg);
  end;
end;

function Point3D(X, Y: Double; Z: Double = 0): TPoint3D;
begin
  Result.X:= X;
  Result.Y:= Y;
  Result.Z:= Z;
end;

function AcadPoint(X, Y: Double; Z: Double = 0): OleVariant;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= X;
  Result[1]:= Y;
  Result[2]:= Z;
end;

function AcadPoint(APoint: TPoint3D): OleVariant; overload;
begin
  Result:= VarArrayCreate([0, 2], varDouble);
  Result[0]:= APoint.X;
  Result[1]:= APoint.Y;
  Result[2]:= APoint.Z;
end;

end.
 