unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, ExtCtrls, AutoCAD_TLB;

type
  TForm1 = class(TForm)
    btnConnect: TButton;
    btnLine: TButton;
    btnCircle: TButton;
    btnArc: TButton;
    btnArrow: TButton;
    GroupBox1: TGroupBox;
    btnPolyLine: TButton;
    sgVertices: TStringGrid;
    btnFill: TButton;
    btn3DPolyline: TButton;
    GroupBox2: TGroupBox;
    rgrpAlignment: TRadioGroup;
    btnText: TButton;
    GroupBox3: TGroupBox;
    rgrpAttachmentPoint: TRadioGroup;
    btnMtext: TButton;
    btnChangeInsPoint: TButton;
    btnEntityInfo: TButton;
    btnSetBulge: TButton;
    GroupBox4: TGroupBox;
    btnAddHatch: TButton;
    btnAddCompoundHatch: TButton;
    btnAddGradientHatch: TButton;
    procedure btnConnectClick(Sender: TObject);
    procedure btnLineClick(Sender: TObject);
    procedure btnCircleClick(Sender: TObject);
    procedure btnArcClick(Sender: TObject);
    procedure btnArrowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgVerticesDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure sgVerticesGetEditMask(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure btnFillClick(Sender: TObject);
    procedure btnPolyLineClick(Sender: TObject);
    procedure btn3DPolylineClick(Sender: TObject);
    procedure btnTextClick(Sender: TObject);
    procedure btnMtextClick(Sender: TObject);
    procedure btnChangeInsPointClick(Sender: TObject);
    procedure btnEntityInfoClick(Sender: TObject);
    procedure btnSetBulgeClick(Sender: TObject);
    procedure btnAddHatchClick(Sender: TObject);
    procedure btnAddGradientHatchClick(Sender: TObject);
    procedure btnAddCompoundHatchClick(Sender: TObject);
  private
    { Private declarations }
    FPoints: array of Double;
    procedure PreparePoints;
    procedure Fill;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses UAutoCAD;

{$R *.dfm}

procedure TForm1.btnConnectClick(Sender: TObject);
begin
  AcConnect(rsDefClassName, False, True);
end;

procedure TForm1.btnLineClick(Sender: TObject);
begin
  if not VarIsClear(Acad) then
  begin
    Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(0, 0), AcadPoint(100, 150));
    Acad.Update;
  end;
end;

procedure TForm1.btnCircleClick(Sender: TObject);
const
  r: Double = 50;
begin
  if not VarIsClear(Acad) then
  begin
    Acad.ActiveDocument.ModelSpace.AddCircle(AcadPoint(0, 0), r);
    Acad.Update;
  end;
end;

procedure TForm1.btnArcClick(Sender: TObject);
const
  r: Double = 50;
  sa: Double = 10 * Pi / 180;
  ea: Double = 190 * Pi / 180;
begin
  if not VarIsClear(Acad) then
  begin
    Acad.ActiveDocument.ModelSpace.AddArc(AcadPoint(0, 0), r, sa, ea);
    Acad.Update;
  end;
end;

procedure TForm1.btnArrowClick(Sender: TObject);
var
  pt, Points, Polyline: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    pt:= AcadPoint(0, 0);
    Points:= VarArrayCreate([0, 5], VarDouble);
    Points[0]:= pt[0];
    Points[1]:= pt[1];
    Points[2]:= pt[0] + 4;
    Points[3]:= pt[1];
    Points[4]:= pt[0] + 10;
    Points[5]:= pt[1];
    Polyline:= Acad.ActiveDocument.ModelSpace.AddLightWeightPolyline(Points);
    Polyline.SetWidth(0, 0, 2);
    Acad.Update;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  sgVertices.ColWidths[0]:= 24;
  Randomize;
  Fill;
end;

procedure TForm1.sgVerticesDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
const
  mFlags = DT_SINGLELINE or DT_LEFT or DT_CENTER or DT_VCENTER;
begin
  with (Sender as TStringGrid).Canvas do
  begin
    if ARow = 0 then
    begin
      case ACol of
        0: DrawText(Handle, 'N', 1, Rect, mFlags);
        1: DrawText(Handle, 'X', 1, Rect, mFlags);
        2: DrawText(Handle, 'Y', 1, Rect, mFlags);
      end;
    end else
    if ACol = 0 then
      DrawText(Handle, PChar(IntToStr(ARow)), -1, Rect, mFlags);
  end;
end;

procedure TForm1.sgVerticesGetEditMask(Sender: TObject; ACol,
  ARow: Integer; var Value: String);
begin
  if (ACol = 1) or (ACol = 2) then
    Value:=  '999';
end;

procedure TForm1.PreparePoints;
var
  i: Integer;
  x, y: Double;
begin
  SetLength(FPoints, 0);
  for i:= 1 to sgVertices.RowCount do
  begin
    try
      x:= StrToFloat(sgVertices.Cells[1, i]);
      y:= StrToFloat(sgVertices.Cells[2, i]);
      SetLength(FPoints, Length(FPoints) + 3);
      FPoints[High(FPoints) - 2]:= x;
      FPoints[High(FPoints) - 1]:= y;
      FPoints[High(FPoints)]:= 0;
    except
      Exit;
    end;
  end;
end;

procedure TForm1.Fill;
var
  i, cnt: Integer;
begin
  cnt:= 2 + Random(14);
  for i:= 1 to cnt do
  begin
    sgVertices.Cells[1, i]:= IntToStr(Random(100));
    sgVertices.Cells[2, i]:= IntToStr(Random(100));
  end;
end;

procedure TForm1.btnFillClick(Sender: TObject);
begin
  Fill;
end;

procedure TForm1.btnPolyLineClick(Sender: TObject);
var
  Points: OleVariant;
  i: Integer;
begin
  if not VarIsClear(Acad) then
  begin
    PreparePoints;
    Points:= VarArrayCreate([0, High(FPoints)], VarDouble);
    for i:= Low(FPoints) to High(FPoints) do
      Points[i]:= FPoints[i];
    SetLength(FPoints, 0);

    Acad.ActiveDocument.ModelSpace.AddPolyline(Points);
    Acad.Update;
  end;
end;

procedure TForm1.btn3DPolylineClick(Sender: TObject);
const
  aSize = 360;
var
  Points: OleVariant;
  i: Integer;
  F, R: Double;
begin
  if not VarIsClear(Acad) then
  begin
    Points:= VarArrayCreate([0, aSize * 3 - 1], VarDouble);
    F:= 0;
    R:= 1;
    i:= 0;
    while i < aSize * 3 do
    begin
      Points[i]:= cos(F) * R;
      Inc(i);
      Points[i]:= sin(F) * R;
      Inc(i);
      Points[i]:= R;
      Inc(i);
      F:= F + 0.1;
      R:= R + 0.1;
    end;
    Acad.ActiveDocument.ModelSpace.Add3Dpoly(Points);
    Acad.Update;
  end;
end;

procedure TForm1.btnTextClick(Sender: TObject);
var
  AcadText: OleVariant;
  AcadCircle: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    AcadText:= Acad.ActiveDocument.ModelSpace.AddText('Hello, World!', AcadPoint(0, 0), 5);
    AcadText.Alignment:= rgrpAlignment.ItemIndex;

    if rgrpAlignment.ItemIndex = 3 then
      AcadText.TextAlignmentPoint:= AcadPoint(20, 0);
    if rgrpAlignment.ItemIndex = 5 then
    begin
      AcadText.TextAlignmentPoint:= AcadPoint(20, 0);
      AcadText.ScaleFactor:= 1;
    end;

    AcadCircle:= Acad.ActiveDocument.ModelSpace.AddCircle(AcadText.InsertionPoint, 1.5);
    AcadCircle.Color:= 1;
    AcadCircle:= Acad.ActiveDocument.ModelSpace.AddCircle(AcadText.TextAlignmentPoint, 1.0);
    AcadCircle.Color:= 2;
    
    Acad.Update;
  end;
end;

var
  AcadMText: OleVariant;

procedure TForm1.btnMtextClick(Sender: TObject);
begin
  if not VarIsClear(Acad) then
  begin
    AcadMText:= Acad.ActiveDocument.ModelSpace.AddMText(AcadPoint(0, 0), 15, 'Hello, World!');
    AcadMText.AttachmentPoint:= rgrpAttachmentPoint.ItemIndex + 1;
    Acad.Update;
  end;
end;

procedure TForm1.btnChangeInsPointClick(Sender: TObject);
begin
  if not VarIsClear(AcadMText) then
  begin
    AcadMText.InsertionPoint:= AcadPoint(10, 0);
    Acad.Update;
  end else
    ShowMessage('MText  !');
end;

procedure TForm1.btnEntityInfoClick(Sender: TObject);
var
  MSpace: OleVariant;
  i, cnt: Integer;
begin
  if not VarIsClear(Acad) then
  begin
    MSpace:= Acad.ActiveDocument.ModelSpace;
    cnt:= MSpace.Count;
    ShowMessage(Format('  %s  %d (-)', [Acad.ActiveDocument.Name, cnt]));
    for i:= 0 to cnt - 1 do
      ShowMessage(Format('EntityName: %s'#13'EntityType: %d', [MSpace.Item(i).EntityName,
                                                               Integer(MSpace.Item(i).EntityType)]));
  end;
end;

procedure TForm1.btnSetBulgeClick(Sender: TObject);
var
  pt, Points, Polyline: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    pt:= AcadPoint(0, 0);
    Points:= VarArrayCreate([0, 9], VarDouble);
    Points[0]:= pt[0];
    Points[1]:= pt[1];
    Points[2]:= pt[0] + 1;
    Points[3]:= pt[1];
    Points[4]:= pt[0] + 1;
    Points[5]:= pt[1] + 0.5;
    Points[6]:= pt[0] + 2;
    Points[7]:= pt[1] + 2.5;
    Points[8]:= pt[0] + 3;
    Points[9]:= pt[1] + 2.5;
    Polyline:= Acad.ActiveDocument.ModelSpace.AddLightWeightPolyline(Points);
    Polyline.SetBulge(2, -0.5);
    Acad.Update;
  end;
end;

procedure TForm1.btnAddHatchClick(Sender: TObject);
var
  Circle, Hatch, Outline: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    Circle:= Acad.ActiveDocument.ModelSpace.AddCircle(AcadPoint(0, 0), 50);
    Outline:= VarArrayCreate([0, 0], varDispatch);
    Outline[0]:= Circle;
    Hatch:= Acad.ActiveDocument.ModelSpace.AddHatch(acHatchPatternTypePreDefined, 'SOLID',
                                                    True, AcHatchObject);

    Hatch.AppendOuterLoop(Outline);
    Hatch.Evaluate;
    Acad.Update;
  end;
end;

procedure TForm1.btnAddGradientHatchClick(Sender: TObject);
var
  Hatch, OuterLoop, InnerLoop, Color1, Color2: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    //   
    OuterLoop:= VarArrayCreate([0, 0], varDispatch);
    OuterLoop[0]:= Acad.ActiveDocument.ModelSpace.AddCircle(AcadPoint(0, 0), 50);
    //   
    InnerLoop:= VarArrayCreate([0, 2], varDispatch);
    InnerLoop[0]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(-26, 15), AcadPoint(0, -30));
    InnerLoop[1]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(0, -30), AcadPoint(26, 15));
    InnerLoop[2]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(26, 15), AcadPoint(-26, 15));
    // 
    Hatch:= Acad.ActiveDocument.ModelSpace.AddHatch(acPreDefinedGradient, 'CYLINDER',
                                                    True, acGradientObject);
    //   
    Color1:= Hatch.GradientColor1;
    Color1.SetRGB(255, 0, 0);
    Hatch.GradientColor1:= Color1;
    Color2:= Hatch.GradientColor2;
    Color2.SetRGB(0, 255, 0);
    Hatch.GradientColor2:= Color2;
    //  
    Hatch.AppendOuterLoop(OuterLoop);
    Hatch.AppendInnerLoop(InnerLoop);
    // 
    Hatch.Evaluate;
    Acad.Update;
  end;
end;

procedure TForm1.btnAddCompoundHatchClick(Sender: TObject);
var
  Hatch, OuterLoop, InnerLoop: OleVariant;
begin
  if not VarIsClear(Acad) then
  begin
    //   
    OuterLoop:= VarArrayCreate([0, 0], varDispatch);
    OuterLoop[0]:= Acad.ActiveDocument.ModelSpace.AddCircle(AcadPoint(0, 0), 50);
    //   
    InnerLoop:= VarArrayCreate([0, 2], varDispatch);
    InnerLoop[0]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(-26, 15), AcadPoint(0, -30));
    InnerLoop[1]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(0, -30), AcadPoint(26, 15));
    InnerLoop[2]:= Acad.ActiveDocument.ModelSpace.AddLine(AcadPoint(26, 15), AcadPoint(-26, 15));
    // 
    Hatch:= Acad.ActiveDocument.ModelSpace.AddHatch(acHatchPatternTypePreDefined, 'SOLID',
                                                    True, AcHatchObject);
    Hatch.AppendOuterLoop(OuterLoop);
    Hatch.AppendInnerLoop(InnerLoop);
    Hatch.Evaluate;
    Acad.Update;
  end;
end;

end.
 